%=========================================================================================
% TUTORIAL 6 to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=========================================================================================
%=========================================================================================

disp('Choose the numerical schemes to be displayed (yes=1 no=0):');
LF = input('Lax-Friedrichs: ');
EO = input('Enquist-Osher : ');
G  = input('Godunov       : ');
U  = input('Upwind        : ');

M = 100;
alfa =  -1;
beta =   1;
cn   = 0.7;
dx     = (beta - alfa)/M; 				
x      = [alfa+0.5*dx:dx:beta-0.5*dx];
n =  0;   %iteration-counter
t =  0;   %time
ns=  0;   %number of selected schemes
NS=['              ';'              ';'              ';'              '];

%--------------------------------------
% initialization
%--------------------------------------
%init = inline('sin(2*pi*(x-alfa)/(beta-alfa))','alfa','beta','x');
%init = inline('(2/3*alfa+1/3*beta <= x & x <= 1/3*alfa+2/3*beta)','alfa','beta','x');
%v0 = init(alfa,beta,x);

for i=1:M,
  v0(i) = exact(alfa,beta,0,x(i));
end;

a  = max(abs(df(v0)));
dt     = cn*dx/a;
lambda = dt/dx;
vmin = min(v0);
vmax = max(v0);

if(LF==1) v_LF = v0;  ns=ns+1;  NS(ns,:) = 'Lax-Friedrichs'; end;
if(EO==1) v_EO = v0;  ns=ns+1;  NS(ns,:) = 'Enquist-Osher '; end;
if(G ==1) v_G  = v0;  ns=ns+1;  NS(ns,:) = 'Godunov       '; end;
if(U ==1) v_U  = v0;  ns=ns+1;  NS(ns,:) = 'Upwind        '; end;

%--------------------------------------
% while-loop
%--------------------------------------
while(1)
  figure(1);clf;hold on;
  for i=1:M, ve(i) = exact(alfa,beta,t,x(i)); end;
  plot([alfa-dx/2 x beta+dx/2],[ve(M) ve ve(1)],'c-');
  if(LF==1) plot([alfa-dx/2 x beta+dx/2],[v_LF(M) v_LF v_LF(1)],'k--'); end;
  if(EO==1) plot([alfa-dx/2 x beta+dx/2],[v_EO(M) v_EO v_EO(1)],'b--'); end;
  if(G ==1) plot([alfa-dx/2 x beta+dx/2],[v_G(M)  v_G  v_G(1) ],'r--'); end;
  if(U ==1) plot([alfa-dx/2 x beta+dx/2],[v_U(M)  v_U  v_U(1) ],'g--'); end;
    %if(EO==1 & G==1)
      %v_D = v_EO - v_G;
      %plot([alfa-dx/2 x beta+dx/2],[v_D(M)  v_D  v_D(1) ],'g-');
    %end;
  title('1D Finite-Volume Schemes');
  text(alfa+0.03*(beta-alfa),0.65*(beta-alfa),['iteration n= ' int2str(n) ]);
  text(alfa+0.03*(beta-alfa),0.60*(beta-alfa),['timestep dt= ' num2str(dt)]);
  text(alfa+0.03*(beta-alfa),0.55*(beta-alfa),['time t = ' num2str(t) ]);
  legend('Exact',NS(1,:),NS(2,:),NS(3,:),NS(4,:));
  axis([alfa beta 0 0.7*(beta-alfa)]);

  N = input('number of performing time-steps ','s');
  if(isempty(N)==1) N=1;
  elseif(N=='q') break; end;
  N = numeric(N);
  if(n+N < 0) continue; end;
  t = t + N*dt;
  n = n + N;
  if(N>=0) it=N;
  else     it=n;
    if(LF==1) v_LF = v0; end;
    if(EO==1) v_EO = v0; end;
    if(G ==1) v_G  = v0; end;
    if(U ==1) v_U  = v0; end;
  end;

  for i=1:it,
    if(LF==1)%Lax-Friedrichs
      v_LF = v_LF - lambda * ( F_LF(lambda,v_LF,[v_LF(2:M) v_LF(1)]) - F_LF(lambda,[v_LF(M) v_LF(1:M-1)],v_LF) );
    end;
    if(EO==1)%Enquist-Osher
      v_EO = v_EO - lambda * ( F_EO(v_EO,[v_EO(2:M) v_EO(1)],0) - F_EO([v_EO(M) v_EO(1:M-1)],v_EO,0) );
    end;
    if(G ==1)%Godunov
      v_G  = v_G  - lambda * ( F_G (v_G ,[v_G(2:M)  v_G(1) ],0) - F_G ([v_G(M)  v_G(1:M-1) ],v_G ,0) );
    end;
      if(U ==1)%combined Upwind (not conservative)
      %v_U  = v_U  - lambda * ( (f(v_U) - f([v_U(M) v_U(1:M-1)])).*(df(v_U)>0) + (f([v_U(2:M) v_U(1)]) - f(v_U)).*(df(v_U)<0) );
      %v_U   = v_U  - lambda * ( F_LU (v_U ,[v_U(2:M)  v_U(1) ]) - F_LU ([v_U(M)  v_U(1:M-1) ],v_U ) );
      v_U   = v_U  - lambda * ( F_RU (v_U ,[v_U(2:M)  v_U(1) ]) - F_RU ([v_U(M)  v_U(1:M-1) ],v_U ) );
    end;
  end;%for
end; %while